use crate::app_config::AppConfig;
use crate::controllers::HttpResult;
use crate::extractors::money_session::MoneySession;
use actix_web::HttpResponse;
use light_openid::primitives::OpenIDConfig;

#[derive(serde::Serialize)]
struct StartOIDCResponse {
    url: String,
}

/// Start OIDC authentication
pub async fn start_oidc(session: MoneySession) -> HttpResult {
    let prov = AppConfig::get().openid_provider();

    let conf = match OpenIDConfig::load_from_url(prov.configuration_url).await {
        Ok(c) => c,
        Err(e) => {
            log::error!("Failed to fetch OpenID provider configuration! {e}");
            return Ok(HttpResponse::InternalServerError()
                .json("Failed to fetch OpenID provider configuration!"));
        }
    };

    let state = match session.gen_oidc_state() {
        Ok(s) => s,
        Err(e) => {
            log::error!("Failed to generate auth state! {e}");
            return Ok(HttpResponse::InternalServerError().json("Failed to generate auth state!"));
        }
    };

    Ok(HttpResponse::Ok().json(StartOIDCResponse {
        url: conf.gen_authorization_url(
            prov.client_id,
            &state,
            &AppConfig::get().oidc_redirect_url(),
        ),
    }))
}

// TODO : take from previous projects
